from dataclasses import dataclass
from typing import TypeVar

from ..result import Result, Success, Failure
from ..transformer import Transformer


L = TypeVar("L")
R = TypeVar("R")
@dataclass
class AndTransform(Transformer[tuple[L, R]]):
    left: Transformer[L]
    right: Transformer[R]

    def parse(self, stream: str, at_index: int) -> Result[tuple[L, R]]:
        result_left = self.left.parse(stream, at_index)
        if isinstance(result_left, Failure):
            return result_left
        result_right = self.right.parse(stream, result_left.next_index)
        if isinstance(result_right, Failure):
            return result_right
        return Success((result_left.value, result_right.value), result_right.next_index)


from okipy.lib import test
from .just import JustTransform


@test()
def test_and_simple(ctx):
    parser = AndTransform(JustTransform("ar"), JustTransform("bre"))
    input = "arbre"
    assert parser.parse(input, 0) == Success(("ar", "bre"), 5)

@test()
def test_and_shifted(ctx):
    parser = AndTransform(JustTransform("ar"), JustTransform("bre"))
    input = "..arbre..."
    assert parser.parse(input, 2) == Success(("ar", "bre"), 7)

@test()
def test_and_none(ctx):
    parser = AndTransform(JustTransform("ar"), JustTransform("bre"))
    input = "......"
    assert parser.parse(input, 2) == Failure(2, set(["ar"]))
