from dataclasses import dataclass
from typing import TypeVar

from ..result import Result, Success, Failure
from ..transformer import Transformer


T = TypeVar("T")
@dataclass
class ListTransform(Transformer[list[T]]):
    item: Transformer[T]

    def parse(self, stream: str, at_index: int) -> Result[list[T]]:
        values = list[T]()
        while True:
            result = self.item.parse(stream, at_index)
            if isinstance(result, Failure):
                break
            values.append(result.value)
            if result.next_index == at_index:
                raise Exception("Parsing empty patterns repeatedly.")
            at_index = result.next_index
        return Success(values, at_index)


from okipy.lib import test
from .just import JustTransform


@test()
def test_list_simple(ctx):
    parser = ListTransform(JustTransform("arbre"))
    input = "arbrearbrearbre"
    assert parser.parse(input, 0) == Success(["arbre", "arbre", "arbre"], 15)

@test()
def test_list_shifted(ctx):
    parser = ListTransform(JustTransform("arbre"))
    input = "..arbrearbre"
    assert parser.parse(input, 2) == Success(["arbre", "arbre"], 12)

@test()
def test_list_none(ctx):
    parser = ListTransform(JustTransform("arbre"))
    input = "......"
    assert parser.parse(input, 2) == Success([], 2)
