from dataclasses import dataclass
from typing import Callable, Generic, TypeVar

from ..result import Result, Success, Failure
from ..transformer import Transformer


I = TypeVar("I")
O = TypeVar("O")
@dataclass
class MapTransform(Generic[I, O], Transformer[O]):
    value: Transformer[I]

    def __init__(self, value: Transformer[I], transform: Callable[[I], O]) -> None:
        self.value = value
        self.transform = transform

    def parse(self, stream: str, at_index: int) -> Result[O]:
        result = self.value.parse(stream, at_index)
        if isinstance(result, Failure):
            return result
        transformed = self.transform(result.value)
        return Success(transformed, result.next_index)


from okipy.lib import test
from .just import JustTransform


@test()
def test_map_simple(ctx):
    parser = MapTransform(JustTransform("arbre"), lambda a: (9, a))
    input = "arbre"
    assert parser.parse(input, 0) == Success((9, "arbre"), 5)

@test()
def test_map_shifted(ctx):
    parser = MapTransform(JustTransform("arbre"), lambda a: (9, a))
    input = "..arbre....."
    assert parser.parse(input, 2) == Success((9, "arbre"), 7)

@test()
def test_map_failure(ctx):
    parser = MapTransform(JustTransform("arbre"), lambda a: (9, a))
    input = "......"
    assert parser.parse(input, 2) == Failure(2, set(["arbre"]))
