from dataclasses import dataclass
from typing import Optional, TypeVar

from ..result import Result, Success
from ..transformer import Transformer


T = TypeVar("T")
@dataclass
class OptionTransform(Transformer[Optional[T]]):
    value: Transformer[T]
    
    def parse(self, stream: str, at_index: int) -> Result[Optional[T]]:
        result = self.value.parse(stream, at_index)
        if isinstance(result, Success):
            return Success(result.value, result.next_index)
        return Success(None, at_index)


from okipy.lib import test
from .just import JustTransform


@test()
def test_opt_simple(ctx):
    parser = OptionTransform(JustTransform("arbre"))
    input = "arbre"
    assert parser.parse(input, 0) == Success("arbre", 5)

@test()
def test_opt_none(ctx):
    parser = OptionTransform(JustTransform("arbre"))
    input = "brear"
    assert parser.parse(input, 0) == Success(None, 0)

@test()
def test_opt_shifted(ctx):
    parser = OptionTransform(JustTransform("arbre"))
    input = "..arbre..."
    assert parser.parse(input, 2) == Success("arbre", 7)

@test()
def test_opt_none_shifted(ctx):
    parser = OptionTransform(JustTransform("arbre"))
    input = "..rbrea..."
    assert parser.parse(input, 2) == Success(None, 2)
