from dataclasses import dataclass
import re

from ..result import Result, Success, Failure
from ..transformer import Transformer


@dataclass
class RegexTransform(Transformer[str]):
    pattern: str

    def parse(self, stream: str, at_index: int) -> Result[str]:
        do_match = re.match(self.pattern, stream[at_index:])
        if do_match is None:
            return Failure(at_index, set([f"<matching '{self.pattern}'>"]))
        else:
            match = do_match[0]
            return Success(match, at_index + len(match))


from okipy.lib import test


@test()
def test_regex_simple(ctx):
    parser = RegexTransform("arbre")
    input = "arbre"
    assert parser.parse(input, 0) == Success("arbre", 5)

@test()
def test_regex_shifted(ctx):
    parser = RegexTransform("arbre")
    input = "..arbre..."
    assert parser.parse(input, 2) == Success("arbre", 7)

@test()
def test_regex_fails(ctx):
    parser = RegexTransform("arbre")
    input = "arbre......"
    assert parser.parse(input, 2) == Failure(2, set(["<matching 'arbre'>"]))

@test()
def test_regex_some(ctx):
    parser = RegexTransform("[0-3]+")
    input = "012345"
    assert parser.parse(input, 0) == Success("0123", 4)
