from dataclasses import dataclass
from typing import TypeVar

from ..result import Result
from ..transformer import Transformer

from .map import MapTransform


I = TypeVar("I")
T = TypeVar("T")
@dataclass
class ValueTransform(Transformer[T]):
    value: T

    def __init__(self, ignored: Transformer[I], value: T):
        self.value = value
        self.actual = MapTransform(ignored, lambda _: value)

    def parse(self, stream: str, at_index: int) -> Result[T]:
        return self.actual.parse(stream, at_index)


from okipy.lib import test
from .just import JustTransform
from ..result import Result, Success, Failure


@test()
def test_value_simple(ctx):
    parser = ValueTransform(JustTransform("arbre"), "Feur")
    input = "arbre"
    assert parser.parse(input, 0) == Success("Feur", 5)

@test()
def test_value_failure(ctx):
    parser = ValueTransform(JustTransform("arbre"), lambda a: (9, a))
    input = "......"
    assert parser.parse(input, 2) == Failure(2, set(["arbre"]))
